@@ -29,13 +29,16 @@ pub use results::*;
 use tantivy::{
     collector::{Collector, TopDocs},
     directory::MmapDirectory,
+    fastfield::FastFieldReader,
     query::{AllQuery, BooleanQuery, FuzzyTermQuery, Occur, PhraseQuery, Query, TermQuery},
     schema::{Field, IndexRecordOption},
-    DocAddress, Document, Index as TantivyIndex, IndexReader, IndexWriter, ReloadPolicy, Searcher,
-    SegmentReader, Term,
+    DocAddress, Document, Index as TantivyIndex, IndexReader, IndexSettings, IndexSortByField,
+    IndexWriter, Order, ReloadPolicy, Searcher, SegmentReader, Term,
 };
 
-use crate::{entity::EntityIdRef, error::Error, ordering::OrderingValueWrapper};
+use crate::{
+    entity::EntityIdRef, error::Error, mutation::OperationId, ordering::OrderingValueWrapper,
+};
 
 mod config;
 mod entity_cache;
@@ -82,7 +85,11 @@ impl MutationIndex {
         let (tantivy_schema, fields) = schema::build_tantivy_schema(config, schemas.as_ref());
 
         let directory = MmapDirectory::open(directory)?;
-        let index = TantivyIndex::open_or_create(directory, tantivy_schema)?;
+        let index = TantivyIndex::builder()
+            .schema(tantivy_schema)
+            .settings(index_settings())
+            .open_or_create(directory)?;
+        fields.register_tokenizers(&index);
 
         fields.register_tokenizers(&index);
 
@@ -117,7 +124,10 @@ impl MutationIndex {
     ) -> Result<MutationIndex, Error> {
         let (tantivy_schema, fields) = schema::build_tantivy_schema(config, schemas.as_ref());
 
-        let index = TantivyIndex::create_in_ram(tantivy_schema);
+        let index = TantivyIndex::builder()
+            .schema(tantivy_schema)
+            .settings(index_settings())
+            .create_in_ram()?;
         fields.register_tokenizers(&index);
 
         let index_reader = index
@@ -212,10 +222,17 @@ impl MutationIndex {
                     let doc = self.entity_tombstone_to_document(&entity_tombstone);
                     index_writer.add_document(doc);
                 }
-                IndexOperation::DeleteOperation(operation_id) => {
-                    trace!("Deleting operation {} from index", operation_id);
-                    index_writer
-                        .delete_term(Term::from_field_u64(self.fields.operation_id, operation_id));
+                IndexOperation::PendingDeletionMarker(entity_id, operation_id) => {
+                    trace!(
+                        "Putting pending deletion marker for entity {} with op {}",
+                        entity_id,
+                        operation_id
+                    );
+
+                    self.entity_cache.remove(&entity_id);
+
+                    let doc = self.pending_deletion_marker_to_document(&entity_id, operation_id);
+                    index_writer.add_document(doc);
                 }
                 IndexOperation::DeleteEntityOperation(entity_id, operation_id) => {
                     trace!(
@@ -674,6 +691,23 @@ impl MutationIndex {
         doc
     }
 
+    /// Converts a marker used to indicate that an entity has pending deletions.
+    /// Only used in pending index, since chain index actually deletes the
+    /// operations.
+    fn pending_deletion_marker_to_document(
+        &self,
+        entity_id: EntityIdRef,
+        operation_id: OperationId,
+    ) -> Document {
+        let mut doc = Document::default();
+
+        doc.add_text(self.fields.entity_id, entity_id);
+        doc.add_u64(self.fields.operation_id, operation_id);
+        doc.add_u64(self.fields.document_type, MutationType::PENDING_DELETION_ID);
+
+        doc
+    }
+
     /// Transforms a text match predicate to Tantivy query.
     fn match_predicate_to_query(
         &self,
@@ -1087,6 +1121,16 @@ impl MutationIndex {
     }
 }
 
+fn index_settings() -> IndexSettings {
+    IndexSettings {
+        sort_by_field: Some(IndexSortByField {
+            field: "operation_id".to_string(),
+            order: Order::Desc,
+        }),
+        ..Default::default()
+    }
+}
+
 fn boost_recent(now_nano: u64, date_nano: u64) -> f32 {
     // From: https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-function-score-query.html#function-decay
     // exp(λ * max(0, |value - origin| - offset))
